import os
import shutil
import pyfbi
from xbrr.edinet.client.document_client import DocumentClient
from xbrr.edinet.reader.reader import Reader
from xbrr.edinet.reader.aspects.finance import Finance


@pyfbi.target
def check():
    _dir = os.path.join(os.path.dirname(__file__), "./data")
    if os.path.exists(_dir):
        shutil.rmtree(_dir)
    else:
        os.mkdir(_dir)
    client = DocumentClient()
    file_path = client.get_xbrl("S100G2KL", save_dir=_dir, expand_level="dir")
    reader = Reader(file_path)
    print("Start Calculation")
    bs = reader.extract(Finance).bs()
    bs.to_csv("bs.csv", index=False, encoding="shift_jis")
    shutil.rmtree(_dir)


with pyfbi.watch():
    check()

pyfbi.dump("result")
pyfbi.show()



from xbrr.base.reader.base_element_value import BaseElementValue


class ElementValue(BaseElementValue):

    def __init__(self, name="", reference="",
                 value="", unit="", decimals="",
                 consolidated=True,
                 context="", member="",
                 period=None, period_start=None,
                 label="", ground=""):
        super().__init__()
        self.name = name
        self.reference = reference
        self.value = value
        self.unit = unit
        self.decimals = decimals
        self.consolidated = consolidated
        self.context = context
        self.member = member
        self.period = period
        self.period_start = period_start
        self.label = label
        self.ground = ground

    @property
    def normalized_text(self):
        return self.normalize(self.value)

    @classmethod
    def create_from_element(cls, reader, element,
                            label_kind="", label_verbose=False):

        _element = element.element
        name = _element.name
        reference = element.reference
        value = _element.text
        unit = ""
        if "unitRef" in _element.attrs:
            unit = _element["unitRef"]

        decimals = ""
        if "decimals" in _element.attrs:
            decimals = _element["decimals"]

        label = ""
        if reader.xbrl_dir and label_kind is not None:
            label = reader\
                    .read_by_link(reference)\
                    .label(label_kind, label_verbose)

        consolidated = True
        period = None
        period_start = None
        context_text = ""
        member = ""

        if "contextRef" in _element.attrs:
            context_id = _element["contextRef"]
            context_ids = context_id.split("_", 1)
            if len(context_ids) > 1:
                context_text = context_ids[0]
                member = context_ids[1]
            else:
                context_text = context_ids[0]

            if "NonConsolidatedMember" in context_id:
                consolidated = False

            context = reader.xbrl.find("xbrli:context", {"id": context_id})
            if context.find("xbrli:instant"):
                period = context.find("xbrli:instant").text
            elif context.find("xbrli:endDate"):
                period = context.find("xbrli:endDate").text
                period_start = context.find("xbrli:startDate").text

        instance = cls(
            name=name, reference=reference,
            value=value, unit=unit, decimals=decimals,
            consolidated=consolidated,
            context=context_text, member=member,
            period=period, period_start=period_start,
            label=label,
            ground=""
        )

        return instance

    def to_dict(self):
        return {
            "name": self.name,
            "reference": self.reference,
            "value": self.value,
            "unit": self.unit,
            "decimals": self.decimals,
            "consolidated": self.consolidated,
            "context": self.context,
            "member": self.member,
            "period": self.period,
            "period_start": self.period_start,
            "label": self.label,
        }



import time
from functools import partial, wraps


def _delay(func, seconds):

    @wraps(func)
    def decorate(*args, **kwargs):
        time.sleep(seconds)
        return func(*args, **kwargs)

    return decorate


delay = partial(_delay, seconds=3)



from xbrr.edinet.client.document_list_client import MetaDataClient
from xbrr.edinet.client.document_list_client import DocumentListClient
from xbrr.edinet.client.document_client import DocumentClient
from xbrr.edinet.reader.reader import Reader
from xbrr.edinet.reader.aspects.business import Business
from xbrr.edinet.reader.aspects.company import Company
from xbrr.edinet.reader.aspects.finance import Finance
from xbrr.edinet.reader.aspects.information import Information
from xbrr.edinet.reader.aspects.metadata import Metadata


class APIFacade():

    @property
    def metadata(self):
        client = MetaDataClient()
        return client

    @property
    def documents(self):
        client = DocumentListClient()
        return client

    @property
    def document(self):
        client = DocumentClient()
        return client


class ReaderFacade():

    def read(self, path):
        return Reader(path)


class AspectFacade():

    @property
    def Business(self):
        return Business

    @property
    def Company(self):
        return Company

    @property
    def Finance(self):
        return Finance

    @property
    def Information(self):
        return Information

    @property
    def Metadata(self):
        return Metadata



from xbrr.edinet.models.metadata import MetaData
from xbrr.edinet.models.document import Document


class Documents():
    """Document lists"""

    def __init__(self, metadata: MetaData = None, documents: list = ()):
        """
        Keyword Arguments:
            count {MetaData} -- Metadata (default: {None}).
            documents {list} -- Document list (default: {()}).
        """
        self.metadata = metadata
        self.__documents = documents

    @property
    def list(self):
        return self.__documents

    @classmethod
    def create(cls, body: dict) -> "Documents":
        """Create instance from EDINET response.

        Arguments:
            body {dict} -- EDINET response.

        Returns:
            Documents -- Documents and its metadata.
        """
        metadata = MetaData.create(body)
        _documents = body["results"]
        documents = [Document.create(d) for d in _documents]

        instance = cls(metadata, documents)
        return instance



class MetaData():
    """Metadata of document list"""

    def __init__(self, count=-1):
        """Metadata of document list

        Keyword Arguments:
            count {int} -- Number of documents (default: {-1}).
        """
        self.count = count

    @classmethod
    def create(cls, body: dict) -> "MetaData":
        """Create instance from EDINET response.

        Arguments:
            body {dict} -- EDINET response.

        Returns:
            MetaData -- Metadata of document list.
        """
        count = body["metadata"]["resultset"]["count"]
        instance = cls(count=count)
        return instance



from datetime import datetime


class Document():

    def __init__(self,
                 sequential_no=1,
                 document_id="S1000001",
                 edinet_code="E10001",
                 sec_code="10000",
                 jcn="0000000000001",
                 filer_name="xx株式会社",
                 fund_code="G00001",
                 ordinance_code="000",
                 form_code="000000",
                 doc_type_code="000",
                 period_start="1000-01-01",
                 period_end="1000-01-01",
                 submitted_date="1000-01-01 12:01",
                 title="",
                 issuer_edinet_code="",
                 subject_edinet_code="",
                 subsidiary_edinet_code="",
                 submit_reason="",
                 parent_document_id="",
                 operated_date="1000-01-01 12:01",
                 withdraw_status=0,
                 operation_status=0,
                 disclosure_status=0,
                 has_xbrl=False,
                 has_pdf=False,
                 has_attachment=False,
                 has_english_doc=False
                 ):
        self.sequential_no = sequential_no
        self.document_id = document_id
        self.edinet_code = edinet_code
        self.sec_code = sec_code
        self.jcn = jcn
        self.filer_name = filer_name
        self.fund_code = fund_code
        self.ordinance_code = ordinance_code
        self.form_code = form_code
        self.doc_type_code = doc_type_code
        self.period_start = period_start
        self.period_end = period_end
        self.submitted_date = submitted_date
        self.title = title
        self.issuer_edinet_code = issuer_edinet_code
        self.subject_edinet_code = subject_edinet_code
        self.subsidiary_edinet_code = subsidiary_edinet_code
        self.submit_reason = submit_reason
        self.parent_document_id = parent_document_id
        self.operated_date = operated_date
        self.withdraw_status = withdraw_status
        self.operation_status = operation_status
        self.disclosure_status = disclosure_status
        self.has_xbrl = has_xbrl
        self.has_pdf = has_pdf
        self.has_attachment = has_attachment
        self.has_english_doc = has_english_doc

    @classmethod
    def create(cls, body: dict) -> "Document":
        def to_date(value, format):
            if value:
                f = "%Y-%m-%d" if format == "ymd" else "%Y-%m-%d %H:%M"
                return datetime.strptime(value, f)
            else:
                return None

        def to_bool(value):
            return True if value == "1" else False

        instance = cls(
            sequential_no=body["seqNumber"],
            document_id=body["docID"],
            edinet_code=body["edinetCode"],
            sec_code=body["secCode"],
            jcn=body["JCN"],
            filer_name=body["filerName"],
            fund_code=body["fundCode"],
            ordinance_code=body["ordinanceCode"],
            form_code=body["formCode"],
            doc_type_code=body["docTypeCode"],
            period_start=to_date(body["periodStart"], "ymd"),
            period_end=to_date(body["periodEnd"], "ymd"),
            submitted_date=to_date(body["submitDateTime"], "ymd_hm"),
            title=body["docDescription"],
            issuer_edinet_code=body["issuerEdinetCode"],
            subject_edinet_code=body["subjectEdinetCode"],
            subsidiary_edinet_code=body["subsidiaryEdinetCode"],
            submit_reason=body["currentReportReason"],
            parent_document_id=body["parentDocID"],
            operated_date=to_date(body["opeDateTime"], "ymd_hm"),
            withdraw_status=int(body["withdrawalStatus"]),
            operation_status=int(body["docInfoEditStatus"]),
            disclosure_status=int(body["disclosureStatus"]),
            has_xbrl=to_bool(body["xbrlFlag"]),
            has_pdf=to_bool(body["pdfFlag"]),
            has_attachment=to_bool(body["attachDocFlag"]),
            has_english_doc=to_bool(body["englishDocFlag"])
        )

        return instance

    @property
    def is_outdated(self):
        if not self.edinet_code and self.withdraw_status == 0:
            return True
        else:
            return False

    @property
    def is_withdrew(self):
        if not self.edinet_code and self.withdraw_status == 2:
            return True
        else:
            return False

    def get_pdf(self, save_dir: str = "", file_name: str = ""):
        from xbrr.edinet.client.document_client import DocumentClient
        client = DocumentClient()
        return client.get_pdf(self.document_id, save_dir, file_name)

    def get_xbrl(self, save_dir: str = "", file_name: str = "",
                 lang: str = "ja",):
        from xbrr.edinet.client.document_client import DocumentClient
        client = DocumentClient()
        return client.get_xbrl(self.document_id, save_dir, file_name, lang)



import os
import unittest
from xbrr.edinet.client.document_client import DocumentClient
from xbrr.edinet.reader.directory import Directory


class TestDirectory(unittest.TestCase):

    def test_xbrl_dir(self):
        _dir = os.path.join(os.path.dirname(__file__), "../data")
        client = DocumentClient()
        file_path = client.get_xbrl("S100FGR9", save_dir=_dir,
                                    expand_level="dir")
        xbrl_dir = Directory(file_path)

        self.assertGreater(len(xbrl_dir.xsd.find_all("element")), 0)
        self.assertGreater(len(xbrl_dir.cal.find_all("calculationLink")), 0)
        self.assertGreater(len(xbrl_dir.def_.find_all("definitionArc")), 0)
        self.assertGreater(len(xbrl_dir.lab.find_all("labelLink")), 0)
        self.assertGreater(len(xbrl_dir.lab_en.find_all("labelLink")), 0)
        self.assertGreater(len(xbrl_dir.pre.find_all("presentationLink")), 0)
        self.assertTrue(xbrl_dir.man.find("manifest"))

        xbrl_dir.delete()
        self.assertFalse(os.path.exists(file_path))



import os
import unittest
from xbrr.edinet.reader.reader import Reader
from xbrr.edinet.reader.aspects.metadata import Metadata


class TestMetadata(unittest.TestCase):

    def get_xbrl(self):
        path = os.path.join(os.path.dirname(__file__),
                            "../../data/xbrl2019.xbrl")
        xbrl = Reader(path)

        return xbrl

    def test_fiscal_year(self):
        xbrl = self.get_xbrl()
        feature = xbrl.extract(Metadata).fiscal_year
        self.assertEqual(feature.value, 2017)

    def test_fiscal_period_kind(self):
        xbrl = self.get_xbrl()
        feature = xbrl.extract(Metadata).fiscal_period_kind
        self.assertEqual(feature.value, "FY")

    def test_company_name(self):
        xbrl = self.get_xbrl()
        feature = xbrl.extract(Metadata).company_name
        self.assertEqual(feature.normalized_text, "TIS株式会社")

    def test_company_name_en(self):
        xbrl = self.get_xbrl()
        feature = xbrl.extract(Metadata).company_name_en
        self.assertEqual(feature.normalized_text, "TIS Inc.")

    def test_address(self):
        xbrl = self.get_xbrl()
        feature = xbrl.extract(Metadata).address
        self.assertEqual(feature.normalized_text, "東京都新宿区西新宿八丁目17番1号")

    def test_phone_number(self):
        xbrl = self.get_xbrl()
        feature = xbrl.extract(Metadata).phone_number
        self.assertEqual(feature.value, "03-5337-7070")



import re
from pathlib import Path
import tempfile
from zipfile import ZipFile
import requests
from xbrr.edinet.client.base_client import BaseClient
from xbrr.edinet.models.error_response import ErrorResponse


class DocumentClient(BaseClient):
    """Client to get file."""

    def __init__(self):
        super().__init__(target="documents/{}")

    def get(self, document_id: str, response_type: str,
            save_dir: str = "", file_name: str = "") -> str:
        """Get file of document_id and save it to save_dir/file_name.

        Arguments:
            document_id {str} -- Document id of EDINET.
            response_type {str} -- Response type of document get API.
                                   "1": Submitted document & audit report (ZIP)
                                   "2": Submitted document (PDF)
                                   "3": Attachment documents (ZIP)
                                   "4": English file (ZIP)

        Keyword Arguments:
            save_dir {str} -- Directory to save file (default: {""}).
            file_name {str} -- Filename of the document (default: {""}).

        Returns:
            str -- Path to saved file.
        """
        url = self.endpoint.format(document_id)
        params = {
            "type": response_type
        }

        # Caution
        r = requests.get(url, params=params, stream=True, verify=False)

        if not r.ok:
            r.raise_for_status()
        elif r.headers["content-type"].startswith("application/json"):
            error = ErrorResponse.create(r.json())
            error.raise_for_status(r)
        else:
            _file_name = file_name
            if not _file_name:
                if "content-disposition" in r.headers:
                    d = r.headers["content-disposition"]
                    file_names = re.findall("filename=\"(.+)\"", d)
                    if len(file_names) > 0:
                        _file_name = file_names[0]

                if not _file_name:
                    ext = ".pdf" if response_type == "2" else ".zip"
                    _file_name = document_id + ext

            chunk_size = 1024
            if save_dir:
                save_path = Path(save_dir).joinpath(_file_name)
            else:
                _file_name = Path(_file_name)
                tmpf = tempfile.NamedTemporaryFile(
                        prefix=_file_name.stem + "__",
                        suffix=_file_name.suffix,
                        delete=False)
                save_path = Path(tmpf.name)

            with save_path.open(mode="wb") as f:
                for chunk in r.iter_content(chunk_size):
                    f.write(chunk)

            return save_path

    def get_pdf(self, document_id: str,
                save_dir: str = "", file_name: str = "") -> str:
        """Get PDF file.

        Arguments:
            document_id {str} -- Document id of EDINET.

        Keyword Arguments:
            save_dir {str} -- Directory to save file (default: {""}).
            file_name {str} -- Filename of the document (default: {""}).

        Returns:
            str -- Saved file path.
        """
        response_type = "2"
        path = self.get(document_id, response_type, save_dir, file_name)
        return path

    def get_xbrl(self, document_id: str,
                 save_dir: str = "", file_name: str = "", lang: str = "ja",
                 expand_level: str = "file"):
        """Get XBRL file.

        Arguments:
            document_id {str} -- Document id of EDINET.

        Keyword Arguments:
            save_dir {str} -- Directory to save file (default: {""}).
            file_name {str} -- Filename of the document (default: {""}).
            lang {str} -- Language of document (default: {"ja"}).
            extract_level {int} -- File expansion level
                                   ''    : Not expand, 
                                   'dir' : Expand zip,
                                   'file': Extract XBRL file from zip.

        Returns:
            str -- Saved file path.
        """
        response_type = -1
        xbrl_path = ""
        if lang == "ja":
            response_type = "1"
            xbrl_path = "XBRL/PublicDoc/"
        elif lang == "en":
            response_type = "4"
            xbrl_path = "XBRL/EnglishDoc/"
        else:
            raise Exception(f"Language {lang} is not supported on EDINET.")

        path = self.get(document_id, response_type, save_dir, file_name)

        if expand_level is None or not expand_level or\
           expand_level not in ("dir", "file"):
            return path
        elif expand_level == "dir":
            xbrl_dir = path.parent.joinpath(document_id)
            with ZipFile(path, "r") as zip:
                zip.extractall(path=xbrl_dir)
            path.unlink()
            return xbrl_dir

        xbrl_path = ""
        with ZipFile(path, "r") as zip:
            files = zip.namelist()
            xbrl_file = ""
            for file_name in files:
                if file_name.startswith(xbrl_path) and \
                   file_name.endswith(".xbrl"):
                    xbrl_file = file_name
                    break

            if xbrl_file:
                xbrl_path = path.with_suffix(".xbrl")
                with xbrl_path.open("wb") as f:
                    f.write(zip.read(xbrl_file))

        if xbrl_path:
            path.unlink()
            path = xbrl_path

        return path

    def get_xbrl_dir(self, document_id: str,
                     save_dir: str = "",
                     file_name: str = "",
                     lang: str = "ja"):
        return self.get_xbrl(
                document_id=document_id,
                save_dir=save_dir,
                file_name=file_name,
                lang=lang,
                expand_level="dir")



import os
import shutil
import unittest
from xbrr.edinet.client.document_client import DocumentClient
from xbrr.edinet.reader.reader import Reader


class TestReader(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        _dir = os.path.join(os.path.dirname(__file__), "../data")
        client = DocumentClient()
        file_path = client.get_xbrl("S100DDYF", save_dir=_dir,
                                    expand_level="dir")
        cls.reader = Reader(file_path)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.reader.xbrl_dir.root)
        if os.path.exists(cls.reader.taxonomy.root):
            shutil.rmtree(cls.reader.taxonomy.root)

    def test_find(self):
        path = os.path.join(os.path.dirname(__file__),
                            "../data/xbrl2019.xbrl")
        xbrl = Reader(path)
        element = xbrl.find("jpdei_cor:EDINETCodeDEI")
        self.assertEqual(element.text, "E05739")

    def test_to_html(self):
        path = os.path.join(os.path.dirname(__file__),
                            "../data/xbrl2019.xbrl")
        xbrl = Reader(path)
        tag = "jpcrp_cor:InformationAboutOfficersTextBlock"
        html = xbrl.find(tag).html

        self.assertTrue(html)

    def test_get_element(self):
        path = os.path.join(os.path.dirname(__file__),
                            "../data/xbrl2019.xbrl")
        xbrl = Reader(path)
        value = xbrl.find("jpcrp_cor:NumberOfEmployees").value()
        print(value.to_dict())
        self.assertEqual(value.value, "19081")
        self.assertEqual(value.decimals, "0")

    def test_taxonomy_year(self):
        self.assertEqual(self.reader.taxonomy_year, "2018")

    def test_roles(self):
        roles = self.reader.roles
        self.assertEqual(len(roles), 3)
        role_names = [r.split("/")[-1] for r in roles]
        self.assertTrue("rol_BalanceSheet" in role_names)
        self.assertTrue("NotesNumber" in role_names)
        self.assertTrue("rol_StatementOfIncome" in role_names)

    def test_namespaces(self):
        roles = self.reader.namespaces
        self.assertEqual(len(roles), 10)

    def test_read_by_link(self):
        taxonomy_element = self.reader.read_by_link("http://disclosure.edinet-fsa.go.jp/taxonomy/jpcrp/2018-03-31/jpcrp_cor_2018-03-31.xsd#jpcrp_cor_AnnexedDetailedScheduleOfProvisionsTextBlock")
        local_element = self.reader.read_by_link("jpcrp030000-asr-001_E00436-000_2018-03-31_01_2018-06-26.xsd#jpcrp030000-asr_E00436-000_ManagementAnalysisOfFinancialPositionOperatingResultsAndCashFlowsHeading")
        self.assertTrue(taxonomy_element)
        self.assertEqual(taxonomy_element.label(), "引当金明細表")
        self.assertTrue(local_element)
        self.assertTrue(local_element.label(), "経営者による財政状態、経営成績及びキャッシュ・フローの状況の分析")

    def test_read_schema_by_role(self):
        bs = self.reader.read_schema_by_role("http://disclosure.edinet-fsa.go.jp/role/jppfs/rol_BalanceSheet")
        self.assertGreater(len(bs), 0)

    def test_read_value_by_role(self):
        pl = self.reader.read_value_by_role("http://disclosure.edinet-fsa.go.jp/role/jppfs/rol_StatementOfIncome")
        self.assertGreater(len(pl), 0)



from datetime import datetime
from xbrr.base.reader.base_parser import BaseParser
from xbrr.edinet.reader.element_value import ElementValue


class Metadata(BaseParser):

    def __init__(self, reader):
        tags = {
            "fiscal_date_start": "jpdei_cor:CurrentFiscalYearStartDateDEI",
            "fiscal_date_end": "jpdei_cor:CurrentFiscalYearEndDateDEI",
            "fiscal_period_kind": "jpdei_cor:TypeOfCurrentPeriodDEI",
            "name": "jpcrp_cor:CompanyNameCoverPage",
            "name_en": "jpcrp_cor:CompanyNameInEnglishCoverPage",
            "address": "jpcrp_cor:AddressOfRegisteredHeadquarterCoverPage",
            "phone_number": "jpcrp_cor:TelephoneNumberAddressOfRegisteredHeadquarterCoverPage"
        }

        super().__init__(reader, ElementValue, tags)

    @property
    def fiscal_year(self):
        value = self.get_text_value("fiscal_date_start")
        if value:
            date = datetime.strptime(value.value, "%Y-%m-%d")
            value.value = date.year
        return value

    @property
    def fiscal_year_end_date(self):
        value = self.get_text_value("fiscal_date_end")
        if value:
            date = datetime.strptime(value.value, "%Y-%m-%d")
            value.value = date
        return value

    @property
    def fiscal_month(self):
        value = self.get_text_value("fiscal_date_start")
        if value:
            date = datetime.strptime(value.value, "%Y-%m-%d")
            value.value = date.month
        return value

    @property
    def fiscal_period_kind(self):
        return self.get_text_value("fiscal_period_kind")

    @property
    def company_name(self):
        return self.get_text_value("name")

    @property
    def company_name_en(self):
        return self.get_text_value("name_en")

    @property
    def address(self):
        return self.get_text_value("address")

    @property
    def phone_number(self):
        return self.get_text_value("phone_number")



import os
import unittest
from xbrr.edinet.models import Document
from tests.utils import delay


class TestDocument(unittest.TestCase):

    @delay
    def test_document_get_xbrl(self):
        _dir = os.path.dirname(__file__)
        d = Document(document_id="S100FGR9")
        path = d.get_xbrl(save_dir=_dir)
        self.assertTrue(os.path.exists(path))
        os.remove(path)

    @delay
    def test_document_get_pdf(self):
        _dir = os.path.dirname(__file__)
        d = Document(document_id="S100FGR9")
        path = d.get_pdf(save_dir=_dir)
        self.assertTrue(os.path.exists(path))
        os.remove(path)



import os
from bs4 import BeautifulSoup


class BaseElement():

    def __init__(self, name, element, reference, reader):
        self.name = name
        self.element = element
        self.reference = reference
        self.reader = reader

    @property
    def reference_path(self):
        names = self.reference.split("#")
        if len(names) > 0:
            return self.reader.link_to_path(names[0])
        else:
            return ""

    @property
    def reference_name(self):
        names = self.reference.split("#")
        if len(names) > 1:
            return names[-1]
        else:
            return ""

    @property
    def exists(self):
        return True if self.element is not None else False

    @property
    def text(self):
        if self.exists:
            return self.element.text
        else:
            return ""

    @property
    def html(self):
        _text = self.text.strip()
        html_text = _text.replace("&lt;", "<").replace("&gt;", ">")
        html = BeautifulSoup(html_text, "html.parser")
        return html

    def _find_file(self, dir, extention):
        path = ""
        for f in os.listdir(dir):
            if f.endswith(extention):
                path = os.path.join(dir, f)
        return path



from requests.exceptions import HTTPError


class ErrorResponse():
    """Error Response."""

    def __init__(self, status: str = "", message: str = ""):
        """
        Keyword Arguments:
            status {str} -- Error Status (default: {""}).
            message {str} -- Error Message (default: {""}).
        """
        self.status = status
        self.message = message

    def raise_for_status(self, response):
        message = f"HTTP Error {self.status}: {self.message}"
        raise HTTPError(message, response=response)

    @classmethod
    def create(cls, body: dict) -> "ErrorResponse":
        """Create instance from EDINET response.

        Arguments:
            body {dict} -- EDINET response.

        Returns:
            ErrorResponse -- Error response.
        """

        metadata = body["metadata"]
        status = metadata["status"]
        message = metadata["message"]

        instance = cls(status, message)
        return instance



from xbrr.base.reader.base_element_schema import BaseElementSchema


class ElementSchema(BaseElementSchema):

    def __init__(self,
                 name="", reference="", label="", alias="",
                 abstract="", data_type="",
                 period_type="", balance=""):
        super().__init__()
        self.name = name
        self.reference = reference
        self.label = label
        self.alias = alias
        self.abstract = abstract
        self.data_type = data_type
        self.period_type = period_type
        self.balance = balance

    def set_alias(self, alias):
        self.alias = alias
        return self

    @classmethod
    def create_from_reference(cls, reader, reference,
                              label_kind="", label_verbose=False):

        name = reference.split("#")[-1]
        label = ""
        abstract = ""
        data_type = ""
        period_type = ""
        balance = ""

        if reader.xbrl_dir:
            _def = reader.read_by_link(reference)
            if label_kind is not None:
                label = _def.label(label_kind, label_verbose)
            xsd = _def.xsd
            abstract = xsd["abstract"]
            data_type = xsd["type"]
            if "xbrli:periodType" in xsd.attrs:
                period_type = xsd["xbrli:periodType"]

            if "xbrli:balance" in xsd.attrs:
                balance = xsd["xbrli:balance"]

        instance = cls(name=name, reference=reference, label=label,
                       abstract=abstract, data_type=data_type,
                       period_type=period_type, balance=balance)
        return instance

    def to_dict(self):
        return {
            "name": self.name,
            "reference": self.reference,
            "label": self.label,
            "abstract": self.abstract,
            "data_type": self.data_type,
            "period_type": self.period_type,
            "balance": self.balance
        }



import os
import re
import unittest
from xbrr.edinet.reader.reader import Reader
from xbrr.base.reader.base_parser import BaseParser
from xbrr.edinet.reader.element import Element


class TestBaseParser(unittest.TestCase):

    def test_search_text(self):
        path = os.path.join(os.path.dirname(__file__),
                            "../data/xbrl2019.xbrl")
        reader = Reader(path)
        tag = "jpcrp_cor:InformationAboutOfficersTextBlock"
        parser = BaseParser(reader, Element, {
            "test": tag
        })
        pattern = "^(男性).+(名).+(女性).+(名)"
        text = parser.search("test", pattern)
        self.assertEqual(text, "男性 13名 女性 1名 (役員のうち女性の比率 7.1%)")

    def test_extract_value(self):
        path = os.path.join(os.path.dirname(__file__),
                            "../data/xbrl2019.xbrl")
        reader = Reader(path)
        tag = "jpcrp_cor:InformationAboutOfficersTextBlock"
        parser = BaseParser(reader, Element, {
            "test": tag
        })
        pattern = "^(男性).+(名).+(女性).+(名)"
        for p, s in [("男性", "名"), ("女性", "名"), ("女性の比率", "%")]:
            value = parser.extract_value("test", p, s, filter_pattern=pattern)
            if p == "男性":
                self.assertEqual(value, 13)
            elif p == "女性":
                self.assertEqual(value, 1)
            else:
                self.assertEqual(value, 7.1)



import os
import time
import unittest
import xbrr
from tests.utils import delay


class TestAPI(unittest.TestCase):

    @delay
    def test_api_metadata(self):
        metadata = xbrr.edinet.api.metadata.get("2019-01-31")
        self.assertGreater(metadata.count, 0)

    @delay
    def test_api_document(self):
        _dir = os.path.dirname(__file__)
        path = xbrr.edinet.api.document.get_pdf("S100FGR9", save_dir=_dir)
        self.assertTrue(os.path.exists(path))
        os.remove(path)

    @delay
    def test_api_documents(self):
        documents = xbrr.edinet.api.documents.get("2019-02-01")
        self.assertEqual(documents.metadata.count, len(documents.list))

        _dir = os.path.dirname(__file__)
        d = documents.list[0]
        for ext in ["xbrl", "pdf"]:
            time.sleep(3)
            if ext == "xbrl":
                path = d.get_xbrl(save_dir=_dir)
            else:
                path = d.get_pdf(save_dir=_dir)

            self.assertTrue(os.path.exists(path))
            os.remove(path)

    def test_extract(self):
        path = os.path.join(os.path.dirname(__file__),
                            "./data/xbrl2019.xbrl")

        result = xbrr.edinet.reader.read(path).extract(
                    xbrr.edinet.aspects.Business).policy_environment_issue_etc
        self.assertTrue(result.value)

    def test_extract_element(self):
        path = os.path.join(os.path.dirname(__file__),
                            "./data/xbrl2019.xbrl")

        result = xbrr.edinet.reader.read(path).extract("information", "number_of_directors")
        self.assertEqual(result.value, 14)



from typing import Union
from datetime import datetime
import requests
from xbrr.edinet.client.base_client import BaseClient
from xbrr.edinet.models import MetaData, Documents


class BaseDocumentListClient(BaseClient):
    """Base client to handle Document List API."""

    def __init__(self, response_type: str):
        """
        Arguments:
            response_type {str} -- Response type of document list api.
        """
        super().__init__(target="documents.json")
        self.response_type = response_type

    def _get(self, date: Union[str, datetime]) -> dict:
        """Get Document List API response.

        Arguments:
            date {(str, datetime)} -- Request date.

        Raises:
            Exception: Date format exception.

        Returns:
            dict -- EDINET Response (JSON).
        """
        url = self.endpoint
        _date = date
        if isinstance(date, str):
            try:
                _date = datetime.strptime(date, "%Y-%m-%d")
            except ValueError:
                raise Exception("Date format should be yyyy-mm-dd.")

        _date = _date.strftime("%Y-%m-%d")

        params = {
            "date": _date,
            "type": self.response_type
        }

        r = requests.get(url, params=params, verify=False)  # Caution

        if not r.ok:
            r.raise_for_status()
        else:
            body = r.json()
            return body


class MetaDataClient(BaseDocumentListClient):
    """Client to get metadata of document list."""

    def __init__(self):
        super().__init__(response_type="1")  # 1 = only metadata.

    def get(self, date: Union[str, datetime]) -> MetaData:
        """Get metadeta response.

        Arguments:
            date {(str, datetime)} -- Request date.

        Returns:
            MetaData -- Metadata of document list.
        """
        body = self._get(date)
        instance = MetaData.create(body)
        return instance


class DocumentListClient(BaseDocumentListClient):
    """Client to get document list."""

    def __init__(self):
        super().__init__(response_type="2")  # 2 = metadata and document list.

    def get(self, date: Union[str, datetime]) -> Documents:
        """Get metadeta response.

        Arguments:
            date {(str, datetime)} -- Request date.

        Returns:
            Documents -- Document list and its metadata.
        """
        body = self._get(date)
        instance = Documents.create(body)
        return instance



import warnings
from xbrr.base.reader.base_parser import BaseParser
from xbrr.edinet.reader.element_value import ElementValue


class Finance(BaseParser):

    def __init__(self, reader):
        tags = {
            "voluntary_accounting_policy_change": "jpcrp_cor:NotesVoluntaryChangesInAccountingPoliciesConsolidatedFinancialStatementsTextBlock",
            "segment_information": "jpcrp_cor:NotesSegmentInformationEtcConsolidatedFinancialStatementsTextBlock",
            "real_estate_for_lease": "jpcrp_cor:NotesRealEstateForLeaseEtcFinancialStatementsTextBlock"
        }

        super().__init__(reader, ElementValue, tags)

    @property
    def use_IFRS(self):
        if "jpigp_cor" in self.reader.namespaces:
            return True
        else:
            return False

    def bs(self, ifrs=False, link_type="calculation"):
        role_uri = "http://disclosure.edinet-fsa.go.jp/role/jppfs/rol_BalanceSheet"
        if ifrs and self.use_IFRS:
            role_uri = "http://disclosure.edinet-fsa.go.jp/role/jpigp/rol_ConsolidatedStatementOfFinancialPositionIFRS"

        bs = self.reader.read_value_by_role(role_uri, link_type)
        return self.__filter_duplicate(bs)

    def pl(self, ifrs=False, link_type="calculation"):
        role_uri = "http://disclosure.edinet-fsa.go.jp/role/jppfs/rol_StatementOfIncome"
        if ifrs and self.use_IFRS:
            role_uri = "http://disclosure.edinet-fsa.go.jp/role/jpigp/rol_ConsolidatedStatementOfComprehensiveIncomeSingleStatementIFRS"
        pl = self.reader.read_value_by_role(role_uri, link_type)
        return self.__filter_duplicate(pl)

    def __filter_duplicate(self, data):
        # Exclude dimension member
        data.drop_duplicates(subset=("name", "period"), keep="first",
                             inplace=True)
        return data

    @property
    def voluntary_accounting_policy_change(self):
        return self.get_text_value("voluntary_accounting_policy_change")

    @property
    def segment_information(self):
        return self.get_text_value("segment_information")

    @property
    def real_estate_for_lease(self):
        return self.get_text_value("real_estate_for_lease")



from setuptools import find_packages, setup


readme = ""
with open("README.md", encoding="utf-8") as f:
    readme = f.read()

setup(
    name="xbrr",
    use_scm_version=True,
    setup_requires=["setuptools_scm"],
    description="eXtensible Business Report Reader.",
    long_description=readme,
    long_description_content_type="text/markdown",
    keywords=["XBRL", "EDINET", "API"],
    author="icoxfog417",
    author_email="icoxfog417@yahoo.co.jp",
    license="MIT",
    packages=find_packages(exclude=('tests',)),
    url="https://github.com/chakki-works/xbrr",
    install_requires=[
        "beautifulsoup4>=4.8.2",
        "joblib>=0.14.1",
        "lxml>=4.4.2",
        "requests>=2.22.0",
        "tqdm>=4.41.1",
        "pandas>=0.25.3"
    ]
)
